@@ -354,22 +354,15 @@ class Parser(ABC):
                         not ref_names - model_names and ref_names - processed_models
                     ):
                         models_to_update += [model.name]
+                        processed_models.add(model.name)
                 imports.append(model.imports)
-                processed_models.add(model.name)
                 for field in model.fields:
                     for data_type in field.data_type.all_data_types:  # type: DataType
-                        if not data_type.type or (
-                            '.' not in data_type.type and data_type.module_name is None
-                        ):
+                        if not data_type.is_modular:
                             continue
-                        type_ = (
-                            f"{data_type.module_name}.{data_type.type}"
-                            if data_type.module_name
-                            else data_type.type
-                        )
-                        from_, import_ = relative(module_path, type_)
-                        full_path = f'{from_}/{import_}'
-                        name = type_.rsplit('.', 1)[-1]
+                        full_name = data_type.full_name
+                        from_, import_ = relative(module_path, full_name)
+                        name = data_type.name
                         if data_type.reference:
                             reference = self.model_resolver.get(
                                 data_type.reference.path
@@ -386,6 +379,7 @@ class Parser(ABC):
                                 if name in model.reference_classes:  # pragma: no cover
                                     model.reference_classes.remove(name)
                                 continue
+                        full_path = f'{from_}/{import_}'
                         if full_path in alias_map:
                             alias = alias_map[full_path] or import_
                         else:
@@ -394,9 +388,9 @@ class Parser(ABC):
                             ).name
                             alias_map[full_path] = None if alias == import_ else alias
                         new_name = f'{alias}.{name}' if from_ and import_ else name
-                        if data_type.module_name and not type_.startswith(from_):
+                        if data_type.module_name and not full_name.startswith(from_):
                             import_map[new_name] = (
-                                f'.{type_[:len(new_name) * - 1 - 1]}',
+                                f'.{full_name[:len(new_name) * - 1 - 1]}',
                                 new_name.split('.')[0],
                             )
                         if name in model.reference_classes:
@@ -407,20 +401,19 @@ class Parser(ABC):
                 for ref_name in model.reference_classes:
                     if ref_name in model_names:
                         continue
-                    if ref_name in import_map:
+                    elif ref_name in import_map:
                         from_, import_ = import_map[ref_name]
                     else:
                         from_, import_ = relative(module_path, ref_name)
                     if init:
                         from_ += "."
-                    if from_ and import_:  # pragma: no cover
-                        imports.append(
-                            Import(
-                                from_=from_,
-                                import_=import_,
-                                alias=alias_map.get(f'{from_}/{import_}'),
-                            )
+                    imports.append(
+                        Import(
+                            from_=from_,
+                            import_=import_,
+                            alias=alias_map.get(f'{from_}/{import_}'),
                         )
+                    )
 
             if with_import:
                 result += [str(imports), str(self.imports), '\n']
